@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Functions to manage the user's Launchpad user ID.
 
@@ -80,7 +80,8 @@ def set_lp_login(username, _config=None):
 def _get_auth_user(auth=None):
     if auth is None:
         auth = AuthenticationConfig()
-    return auth.get_user('ssh', '.launchpad.net')
+    username = auth.get_user('ssh', '.launchpad.net')
+    return username
 
 def _set_auth_user(username, auth=None):
     if auth is None:
