@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,19 +14,68 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Reconfigure a bzrdir into a new tree/branch/repository layout"""
+"""Reconfigure a bzrdir into a new tree/branch/repository layout.
+
+Various types of reconfiguration operation are available either by
+constructing a class or using a factory method on Reconfigure.
+"""
+
 
 from bzrlib import (
     branch,
     bzrdir,
     errors,
+    trace,
+    ui,
+    urlutils,
     )
 
+
+# TODO: common base class for all reconfigure operations, making no
+# assumptions about what kind of change will be done.
+
+
+class ReconfigureStackedOn(object):
+    """Reconfigures a branch to be stacked on another branch."""
+
+    def apply(self, bzrdir, stacked_on_url):
+        branch = bzrdir.open_branch()
+        # it may be a path relative to the cwd or a url; the branch wants
+        # a path relative to itself...
+        on_url = urlutils.relative_url(branch.base,
+            urlutils.normalize_url(stacked_on_url))
+        branch.lock_write()
+        try:
+            branch.set_stacked_on_url(on_url)
+            if not trace.is_quiet():
+                ui.ui_factory.note(
+                    "%s is now stacked on %s\n"
+                    % (branch.base, branch.get_stacked_on_url()))
+        finally:
+            branch.unlock()
+
+
+class ReconfigureUnstacked(object):
+
+    def apply(self, bzrdir):
+        branch = bzrdir.open_branch()
+        branch.lock_write()
+        try:
+            branch.set_stacked_on_url(None)
+            if not trace.is_quiet():
+                ui.ui_factory.note(
+                    "%s is now not stacked\n"
+                    % (branch.base,))
+        finally:
+            branch.unlock()
+
+
 class Reconfigure(object):
 
     def __init__(self, bzrdir, new_bound_location=None):
         self.bzrdir = bzrdir
         self.new_bound_location = new_bound_location
+        self.local_repository = None
         try:
             self.repository = self.bzrdir.find_repository()
         except errors.NoRepositoryPresent:
@@ -217,8 +266,8 @@ class Reconfigure(object):
     def _check(self):
         """Raise if reconfiguration would destroy local changes"""
         if self._destroy_tree:
-            changes = self.tree.changes_from(self.tree.basis_tree())
-            if changes.has_changed():
+            # XXX: What about pending merges ? -- vila 20090629
+            if self.tree.has_changes(self.tree.basis_tree()):
                 raise errors.UncommittedChanges(self.tree)
         if self._create_reference and self.local_branch is not None:
             reference_branch = branch.Branch.open(self._select_bind_location())
