@@ -33,23 +33,6 @@ from bzrlib.errors import NotBranchError
 
 class TestExceptionReporting(TestCase):
 
-    def test_report_exception(self):
-        """When an error occurs, display bug report details to stderr"""
-        try:
-            raise AssertionError("failed")
-        except AssertionError, e:
-            erf = StringIO()
-            trace.report_exception(sys.exc_info(), erf)
-        err = erf.getvalue()
-        self.assertContainsRe(err,
-            r'bzr: ERROR: exceptions\.AssertionError: failed\n')
-        self.assertContainsRe(err,
-            r'Please report a bug at https://bugs\.launchpad\.net/bzr/\+filebug')
-        self.assertContainsRe(err,
-            '(?m)^encoding: .*, fsenc: .*, lang: .*')
-        self.assertContainsRe(err,
-            '(?m)^plugins:$')
-
     def test_exception_exitcode(self):
         # we must use a subprocess, because the normal in-memory mechanism
         # allows errors to propagate up through the test suite
@@ -58,8 +41,8 @@ class TestExceptionReporting(TestCase):
             retcode=errors.EXIT_INTERNAL_ERROR)
         self.assertEqual(4, errors.EXIT_INTERNAL_ERROR)
         self.assertContainsRe(err,
-                r'bzr: ERROR: exceptions\.AssertionError: always fails\n')
-        self.assertContainsRe(err, r'Please report a bug at')
+                r'exceptions\.AssertionError: always fails\n')
+        self.assertContainsRe(err, r'Bazaar has encountered an internal error')
 
 
 class TestDeprecationWarning(TestCaseInTempDir):
