@@ -368,8 +368,12 @@ class TestPull(ExternalBase):
         See <https://launchpad.net/bugs/380314>
         """
         self.setup_smart_server_with_call_log()
+        # Make a stacked-on branch with two commits so that the
+        # revision-history can't be determined just by looking at the parent
+        # field in the revision in the stacked repo.
         parent = self.make_branch_and_tree('parent', format='1.9')
         parent.commit(message='first commit')
+        parent.commit(message='second commit')
         local = parent.bzrdir.sprout('local').open_workingtree()
         local.commit(message='local commit')
         local.branch.create_clone_on_transport(
@@ -383,7 +387,7 @@ class TestPull(ExternalBase):
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(43, self.hpss_calls)
+        self.assertLength(18, self.hpss_calls)
         remote = Branch.open('stacked')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 
