@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
 import socket
@@ -33,6 +33,7 @@ from bzrlib import (
     errors,
     tests,
     transport as _mod_transport,
+    ui,
     )
 from bzrlib.osutils import (
     pathjoin,
@@ -546,3 +547,12 @@ class TestUsesAuthConfig(TestCaseWithSFTPServer):
     def test_sftp_is_none_if_no_config(self):
         t = self.get_transport_for_connection(set_config=False)
         self.assertIs(None, t._get_credentials()[0])
+
+    def test_sftp_doesnt_prompt_username(self):
+        stdout = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdin='joe\nfoo\n', stdout=stdout)
+        t = self.get_transport_for_connection(set_config=False)
+        self.assertIs(None, t._get_credentials()[0])
+        # No prompts should've been printed, stdin shouldn't have been read
+        self.assertEquals("", stdout.getvalue())
+        self.assertEquals(0, ui.ui_factory.stdin.tell())
