@@ -40,17 +40,21 @@ EMPTY_SHELF = ("Bazaar pack format 1 (introduced in 0.18)\n"
 
 class TestPrepareShelf(tests.TestCaseWithTransport):
 
-    def test_shelve_rename(self):
+    def prepare_shelve_rename(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['foo'])
         tree.add(['foo'], ['foo-id'])
         tree.commit('foo')
         tree.rename_one('foo', 'bar')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('rename', 'foo-id', 'foo', 'bar')],
                           list(creator.iter_shelvable()))
-        creator.shelve_rename('foo-id')
+        return creator
+
+    def check_shelve_rename(self, creator):
         work_trans_id = creator.work_transform.trans_id_file_id('foo-id')
         self.assertEqual('foo', creator.work_transform.final_name(
                          work_trans_id))
@@ -58,17 +62,31 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertEqual('bar', creator.shelf_transform.final_name(
                          shelf_trans_id))
 
-    def test_shelve_move(self):
+    def test_shelve_rename(self):
+        creator = self.prepare_shelve_rename()
+        creator.shelve_rename('foo-id')
+        self.check_shelve_rename(creator)
+
+    def test_shelve_change_handles_rename(self):
+        creator = self.prepare_shelve_rename()
+        creator.shelve_change(('rename', 'foo-id', 'foo', 'bar'))
+        self.check_shelve_rename(creator)
+
+    def prepare_shelve_move(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['foo/', 'bar/', 'foo/baz'])
         tree.add(['foo', 'bar', 'foo/baz'], ['foo-id', 'bar-id', 'baz-id'])
         tree.commit('foo')
         tree.rename_one('foo/baz', 'bar/baz')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('rename', 'baz-id', 'foo/baz', 'bar/baz')],
                          list(creator.iter_shelvable()))
-        creator.shelve_rename('baz-id')
+        return creator, tree
+
+    def check_shelve_move(self, creator, tree):
         work_trans_id = creator.work_transform.trans_id_file_id('baz-id')
         work_foo = creator.work_transform.trans_id_file_id('foo-id')
         self.assertEqual(work_foo, creator.work_transform.final_parent(
@@ -80,12 +98,22 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         creator.transform()
         self.assertEqual('foo/baz', tree.id2path('baz-id'))
 
+    def test_shelve_move(self):
+        creator, tree = self.prepare_shelve_move()
+        creator.shelve_rename('baz-id')
+        self.check_shelve_move(creator, tree)
+
+    def test_shelve_change_handles_move(self):
+        creator, tree = self.prepare_shelve_move()
+        creator.shelve_change(('rename', 'baz-id', 'foo/baz', 'bar/baz'))
+        self.check_shelve_move(creator, tree)
+
     def assertShelvedFileEqual(self, expected_content, creator, file_id):
         s_trans_id = creator.shelf_transform.trans_id_file_id(file_id)
         shelf_file = creator.shelf_transform._limbo_name(s_trans_id)
         self.assertFileEqual(expected_content, shelf_file)
 
-    def test_shelve_content_change(self):
+    def prepare_content_change(self):
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -95,6 +123,10 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.build_tree_contents([('foo', 'b\na\nc\n')])
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
+        return creator
+
+    def test_shelve_content_change(self):
+        creator = self.prepare_content_change()
         self.assertEqual([('modify text', 'foo-id')],
                          list(creator.iter_shelvable()))
         creator.shelve_lines('foo-id', ['a\n', 'c\n'])
@@ -102,7 +134,21 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertFileEqual('a\nc\n', 'foo')
         self.assertShelvedFileEqual('b\na\n', creator, 'foo-id')
 
-    def test_shelve_creation(self):
+    def test_shelve_change_handles_modify_text(self):
+        creator = self.prepare_content_change()
+        creator.shelve_change(('modify text', 'foo-id'))
+        creator.transform()
+        self.assertFileEqual('a\n', 'foo')
+        self.assertShelvedFileEqual('b\na\nc\n', creator, 'foo-id')
+
+    def test_shelve_all(self):
+        creator = self.prepare_content_change()
+        creator.shelve_all()
+        creator.transform()
+        self.assertFileEqual('a\n', 'foo')
+        self.assertShelvedFileEqual('b\na\nc\n', creator, 'foo-id')
+
+    def prepare_shelve_creation(self):
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -114,9 +160,9 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertEqual([('add file', 'bar-id', 'directory', 'bar'),
                           ('add file', 'foo-id', 'file', 'foo')],
                           sorted(list(creator.iter_shelvable())))
-        creator.shelve_creation('foo-id')
-        creator.shelve_creation('bar-id')
-        creator.transform()
+        return creator, tree
+
+    def check_shelve_creation(self, creator, tree):
         self.assertRaises(StopIteration,
                           tree.iter_entries_by_dir(['foo-id']).next)
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
@@ -129,7 +175,22 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertEqual('directory',
             creator.shelf_transform.final_kind(s_bar_trans_id))
 
-    def _test_shelve_symlink_creation(self, link_name, link_target):
+    def test_shelve_creation(self):
+        creator, tree = self.prepare_shelve_creation()
+        creator.shelve_creation('foo-id')
+        creator.shelve_creation('bar-id')
+        creator.transform()
+        self.check_shelve_creation(creator, tree)
+
+    def test_shelve_change_handles_creation(self):
+        creator, tree = self.prepare_shelve_creation()
+        creator.shelve_change(('add file', 'foo-id', 'file', 'foo'))
+        creator.shelve_change(('add file', 'bar-id', 'directory', 'bar'))
+        creator.transform()
+        self.check_shelve_creation(creator, tree)
+
+    def _test_shelve_symlink_creation(self, link_name, link_target,
+                                      shelve_change=False):
         self.requireFeature(tests.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
@@ -141,7 +202,10 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.addCleanup(creator.finalize)
         self.assertEqual([('add file', 'foo-id', 'symlink', link_name)],
                          list(creator.iter_shelvable()))
-        creator.shelve_creation('foo-id')
+        if shelve_change:
+            creator.shelve_change(('add file', 'foo-id', 'symlink', link_name))
+        else:
+            creator.shelve_creation('foo-id')
         creator.transform()
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
         self.failIfExists(link_name)
@@ -158,8 +222,12 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self._test_shelve_symlink_creation(u'fo\N{Euro Sign}o',
                                            u'b\N{Euro Sign}ar')
 
+    def test_shelve_change_handles_symlink_creation(self):
+        self._test_shelve_symlink_creation('foo', 'bar', shelve_change=True)
+
     def _test_shelve_symlink_target_change(self, link_name,
-                                           old_target, new_target):
+                                           old_target, new_target,
+                                           shelve_change=False):
         self.requireFeature(tests.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
@@ -174,7 +242,11 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertEqual([('modify target', 'foo-id', link_name,
                            old_target, new_target)],
                          list(creator.iter_shelvable()))
-        creator.shelve_modify_target('foo-id')
+        if shelve_change:
+            creator.shelve_change(('modify target', 'foo-id', link_name,
+                                   old_target, new_target))
+        else:
+            creator.shelve_modify_target('foo-id')
         creator.transform()
         self.assertEqual(old_target, osutils.readlink(link_name))
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
@@ -191,6 +263,10 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self._test_shelve_symlink_target_change(
             u'fo\N{Euro Sign}o', u'b\N{Euro Sign}ar', u'b\N{Euro Sign}az')
 
+    def test_shelve_change_handles_symlink_target_change(self):
+        self._test_shelve_symlink_target_change('foo', 'bar', 'baz',
+                                                shelve_change=True)
+
     def test_shelve_creation_no_contents(self):
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
@@ -213,7 +289,7 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
                          creator.shelf_transform.final_file_id(s_trans_id))
         self.failIfExists('foo')
 
-    def test_shelve_deletion(self):
+    def prepare_shelve_deletion(self):
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -228,19 +304,35 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         self.assertEqual([('delete file', 'bar-id', 'file', 'foo/bar'),
                           ('delete file', 'foo-id', 'directory', 'foo')],
                           sorted(list(creator.iter_shelvable())))
-        creator.shelve_deletion('foo-id')
-        creator.shelve_deletion('bar-id')
-        creator.transform()
+        return creator, tree
+
+    def check_shelve_deletion(self, tree):
         self.assertTrue('foo-id' in tree)
         self.assertTrue('bar-id' in tree)
         self.assertFileEqual('baz', 'tree/foo/bar')
 
+    def test_shelve_deletion(self):
+        creator, tree = self.prepare_shelve_deletion()
+        creator.shelve_deletion('foo-id')
+        creator.shelve_deletion('bar-id')
+        creator.transform()
+        self.check_shelve_deletion(tree)
+
+    def test_shelve_change_handles_deletion(self):
+        creator, tree = self.prepare_shelve_deletion()
+        creator.shelve_change(('delete file', 'foo-id', 'directory', 'foo'))
+        creator.shelve_change(('delete file', 'bar-id', 'file', 'foo/bar'))
+        creator.transform()
+        self.check_shelve_deletion(tree)
+
     def test_shelve_delete_contents(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo',])
         tree.add('foo', 'foo-id')
         tree.commit('Added file and directory')
         os.unlink('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('delete file', 'foo-id', 'file', 'foo')],
@@ -249,30 +341,57 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         creator.transform()
         self.failUnlessExists('tree/foo')
 
-    def test_shelve_change_kind(self):
+    def prepare_shelve_change_kind(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/foo', 'bar')])
         tree.add('foo', 'foo-id')
         tree.commit('Added file and directory')
         os.unlink('tree/foo')
         os.mkdir('tree/foo')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('change kind', 'foo-id', 'file', 'directory',
                            'foo')], sorted(list(creator.iter_shelvable())))
-        creator.shelve_content_change('foo-id')
-        creator.transform()
+        return creator
+
+    def check_shelve_change_kind(self, creator):
         self.assertFileEqual('bar', 'tree/foo')
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
         self.assertEqual('directory',
                          creator.shelf_transform._new_contents[s_trans_id])
 
+    def test_shelve_change_kind(self):
+        creator = self.prepare_shelve_change_kind()
+        creator.shelve_content_change('foo-id')
+        creator.transform()
+        self.check_shelve_change_kind(creator)
+
+    def test_shelve_change_handles_change_kind(self):
+        creator = self.prepare_shelve_change_kind()
+        creator.shelve_change(('change kind', 'foo-id', 'file', 'directory',
+                               'foo'))
+        creator.transform()
+        self.check_shelve_change_kind(creator)
+
+    def test_shelve_change_unknown_change(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        e = self.assertRaises(ValueError, creator.shelve_change, ('unknown',))
+        self.assertEqual('Unknown change kind: "unknown"', str(e))
+
     def test_shelve_unversion(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo',])
         tree.add('foo', 'foo-id')
         tree.commit('Added file and directory')
         tree.unversion(['foo-id'])
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         self.assertEqual([('delete file', 'foo-id', 'file', 'foo')],
@@ -283,6 +402,8 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
 
     def test_shelve_serialization(self):
         tree = self.make_branch_and_tree('.')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         shelf_file = open('shelf', 'wb')
@@ -297,6 +418,8 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/foo'])
         tree.add('foo', 'foo-id')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
         self.addCleanup(creator.finalize)
         list(creator.iter_shelvable())
@@ -321,8 +444,12 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
 
     def test_shelve_unversioned(self):
         tree = self.make_branch_and_tree('tree')
-        self.assertRaises(errors.PathsNotVersionedError,
-                          shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        tree.lock_tree_write()
+        try:
+            self.assertRaises(errors.PathsNotVersionedError,
+                              shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        finally:
+            tree.unlock()
         # We should be able to lock/unlock the tree if ShelfCreator cleaned
         # after itself.
         wt = workingtree.WorkingTree.open('tree')
@@ -330,8 +457,28 @@ class TestPrepareShelf(tests.TestCaseWithTransport):
         wt.unlock()
         # And a second tentative should raise the same error (no
         # limbo/pending_deletion leftovers).
-        self.assertRaises(errors.PathsNotVersionedError,
-                          shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        tree.lock_tree_write()
+        try:
+            self.assertRaises(errors.PathsNotVersionedError,
+                              shelf.ShelfCreator, tree, tree.basis_tree(), ['foo'])
+        finally:
+            tree.unlock()
+
+    def test_shelve_skips_added_root(self):
+        """Skip adds of the root when iterating through shelvable changes."""
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        self.assertEqual([], list(creator.iter_shelvable()))
+
+    def test_shelve_skips_added_root(self):
+        """Skip adds of the root when iterating through shelvable changes."""
+        tree = self.make_branch_and_tree('tree')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        self.assertEqual([], list(creator.iter_shelvable()))
 
 
 class TestUnshelver(tests.TestCaseWithTransport):
@@ -435,6 +582,7 @@ class TestUnshelver(tests.TestCaseWithTransport):
         shelf_file = open('shelf', 'rb')
         self.addCleanup(shelf_file.close)
         unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
+        unshelver.finalize()
 
     def test_corrupt_shelf(self):
         tree = self.make_branch_and_tree('.')
@@ -554,7 +702,10 @@ class TestShelfManager(tests.TestCaseWithTransport):
 
     def test_get_metadata(self):
         tree = self.make_branch_and_tree('.')
+        tree.lock_tree_write()
+        self.addCleanup(tree.unlock)
         creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
         shelf_manager = tree.get_shelf_manager()
         shelf_id = shelf_manager.shelve_changes(creator, 'foo')
         metadata = shelf_manager.get_metadata(shelf_id)
