@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,11 +12,11 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Implementation of Transport that adapts another transport to be readonly."""
 
-from bzrlib.errors import TransportNotPossible, NoSmartServer, NoSmartMedium
+from bzrlib.errors import TransportNotPossible, NoSmartMedium
 from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
 
 
