@@ -15,10 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-
 """Text UI, write output to the console.
 """
 
+import getpass
+import os
 import sys
 import time
 import warnings
@@ -26,6 +27,7 @@ import warnings
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    debug,
     progress,
     osutils,
     symbol_versioning,
@@ -33,31 +35,33 @@ from bzrlib import (
 
 """)
 
-from bzrlib.ui import CLIUIFactory
+from bzrlib.ui import (
+    UIFactory,
+    NullProgressView,
+    )
 
 
-class TextUIFactory(CLIUIFactory):
+class TextUIFactory(UIFactory):
     """A UI factory for Text user interefaces."""
 
     def __init__(self,
-                 bar_type=None,
                  stdin=None,
                  stdout=None,
                  stderr=None):
         """Create a TextUIFactory.
 
-        :param bar_type: The type of progress bar to create. It defaults to
-                         letting the bzrlib.progress.ProgressBar factory auto
-                         select.   Deprecated.
+        :param bar_type: The type of progress bar to create.  Deprecated
+            and ignored; a TextProgressView is always used.
         """
-        super(TextUIFactory, self).__init__(stdin=stdin,
-                stdout=stdout, stderr=stderr)
-        if bar_type:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 11, 0))
-                % "bar_type parameter")
+        super(TextUIFactory, self).__init__()
+        # TODO: there's no good reason not to pass all three streams, maybe we
+        # should deprecate the default values...
+        self.stdin = stdin
+        self.stdout = stdout
+        self.stderr = stderr
         # paints progress, network activity, etc
-        self._progress_view = TextProgressView(self.stderr)
-
+        self._progress_view = self.make_progress_view()
+        
     def clear_term(self):
         """Prepare the terminal for output.
 
@@ -69,18 +73,107 @@ class TextUIFactory(CLIUIFactory):
         # to clear it.  We might need to separately check for the case of
         self._progress_view.clear()
 
+    def get_boolean(self, prompt):
+        while True:
+            self.prompt(prompt + "? [y/n]: ")
+            line = self.stdin.readline().lower()
+            if line in ('y\n', 'yes\n'):
+                return True
+            elif line in ('n\n', 'no\n'):
+                return False
+            elif line in ('', None):
+                # end-of-file; possibly should raise an error here instead
+                return None
+
+    def get_non_echoed_password(self):
+        isatty = getattr(self.stdin, 'isatty', None)
+        if isatty is not None and isatty():
+            # getpass() ensure the password is not echoed and other
+            # cross-platform niceties
+            password = getpass.getpass('')
+        else:
+            # echo doesn't make sense without a terminal
+            password = self.stdin.readline()
+            if not password:
+                password = None
+            elif password[-1] == '\n':
+                password = password[:-1]
+        return password
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user
+                 canceled the request.
+        """
+        prompt += ': '
+        self.prompt(prompt, **kwargs)
+        # There's currently no way to say 'i decline to enter a password'
+        # as opposed to 'my password is empty' -- does it matter?
+        return self.get_non_echoed_password()
+
+    def get_username(self, prompt, **kwargs):
+        """Prompt the user for a username.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The username string, return None if the user
+                 canceled the request.
+        """
+        prompt += ': '
+        self.prompt(prompt, **kwargs)
+        username = self.stdin.readline()
+        if not username:
+            username = None
+        elif username[-1] == '\n':
+            username = username[:-1]
+        return username
+
+    def make_progress_view(self):
+        """Construct and return a new ProgressView subclass for this UI.
+        """
+        # if the user specifically requests either text or no progress bars,
+        # always do that.  otherwise, guess based on $TERM and tty presence.
+        if os.environ.get('BZR_PROGRESS_BAR') == 'text':
+            return TextProgressView(self.stderr)
+        elif os.environ.get('BZR_PROGRESS_BAR') == 'none':
+            return NullProgressView()
+        elif progress._supports_progress(self.stderr):
+            return TextProgressView(self.stderr)
+        else:
+            return NullProgressView()
+
     def note(self, msg):
         """Write an already-formatted message, clearing the progress bar if necessary."""
         self.clear_term()
         self.stdout.write(msg + '\n')
 
+    def prompt(self, prompt, **kwargs):
+        """Emit prompt on the CLI.
+        
+        :param kwargs: Dictionary of arguments to insert into the prompt,
+            to allow UIs to reformat the prompt.
+        """
+        if kwargs:
+            # See <https://launchpad.net/bugs/365891>
+            prompt = prompt % kwargs
+        prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
+        self.clear_term()
+        self.stderr.write(prompt)
+
     def report_transport_activity(self, transport, byte_count, direction):
         """Called by transports as they do IO.
 
         This may update a progress bar, spinner, or similar display.
         By default it does nothing.
         """
-        self._progress_view._show_transport_activity(transport,
+        self._progress_view.show_transport_activity(transport,
             direction, byte_count)
 
     def _progress_updated(self, task):
@@ -118,6 +211,7 @@ class TextProgressView(object):
         # true when there's output on the screen we may need to clear
         self._have_output = False
         # XXX: We could listen for SIGWINCH and update the terminal width...
+        # https://launchpad.net/bugs/316357
         self._width = osutils.terminal_width()
         self._last_transport_msg = ''
         self._spin_pos = 0
@@ -128,8 +222,10 @@ class TextProgressView(object):
         self._last_task = None
         self._total_byte_count = 0
         self._bytes_since_update = 0
+        self._fraction = 0
 
     def _show_line(self, s):
+        # sys.stderr.write("progress %r\n" % s)
         n = self._width - 1
         self._term_file.write('\r%-*.*s\r' % (n, n, s))
 
@@ -151,9 +247,14 @@ class TextProgressView(object):
             cols = 20
             if self._last_task is None:
                 completion_fraction = 0
+                self._fraction = 0
             else:
                 completion_fraction = \
                     self._last_task._overall_completion_fraction() or 0
+            if (completion_fraction < self._fraction and 'progress' in
+                debug.debug_flags):
+                import pdb;pdb.set_trace()
+            self._fraction = completion_fraction
             markers = int(round(float(cols) * completion_fraction)) - 1
             bar_str = '[' + ('#' * markers + spin_str).ljust(cols) + '] '
             return bar_str
@@ -189,9 +290,12 @@ class TextProgressView(object):
             task_msg = self._format_task(self._last_task)
         else:
             task_msg = ''
-        trans = self._last_transport_msg
-        if trans:
-            trans += ' | '
+        if self._last_task and not self._last_task.show_transport_activity:
+            trans = ''
+        else:
+            trans = self._last_transport_msg
+            if trans:
+                trans += ' | '
         return (bar_string + trans + task_msg)
 
     def _repaint(self):
@@ -208,7 +312,7 @@ class TextProgressView(object):
         must_update = task is not self._last_task
         self._last_task = task
         now = time.time()
-        if (not must_update) and (now < self._last_repaint + 0.1):
+        if (not must_update) and (now < self._last_repaint + task.update_latency):
             return
         if now > self._transport_update_time + 10:
             # no recent activity; expire it
@@ -216,7 +320,7 @@ class TextProgressView(object):
         self._last_repaint = now
         self._repaint()
 
-    def _show_transport_activity(self, transport, direction, byte_count):
+    def show_transport_activity(self, transport, direction, byte_count):
         """Called by transports via the ui_factory, as they do IO.
 
         This may update a progress bar, spinner, or similar display.
@@ -225,28 +329,31 @@ class TextProgressView(object):
         # XXX: Probably there should be a transport activity model, and that
         # too should be seen by the progress view, rather than being poked in
         # here.
+        if not self._have_output:
+            # As a workaround for <https://launchpad.net/bugs/321935> we only
+            # show transport activity when there's already a progress bar
+            # shown, which time the application code is expected to know to
+            # clear off the progress bar when it's going to send some other
+            # output.  Eventually it would be nice to have that automatically
+            # synchronized.
+            return
         self._total_byte_count += byte_count
         self._bytes_since_update += byte_count
         now = time.time()
+        if self._total_byte_count < 2000:
+            # a little resistance at first, so it doesn't stay stuck at 0
+            # while connecting...
+            return
         if self._transport_update_time is None:
             self._transport_update_time = now
         elif now >= (self._transport_update_time + 0.5):
             # guard against clock stepping backwards, and don't update too
             # often
             rate = self._bytes_since_update / (now - self._transport_update_time)
-            scheme = getattr(transport, '_scheme', None) or repr(transport)
-            if direction == 'read':
-                dir_char = '>'
-            elif direction == 'write':
-                dir_char = '<'
-            else:
-                dir_char = ' '
-            msg = ("%.7s %s %6dKB %5dKB/s" %
-                    (scheme, dir_char, self._total_byte_count>>10, int(rate)>>10,))
+            msg = ("%6dKB %5dKB/s" %
+                    (self._total_byte_count>>10, int(rate)>>10,))
             self._transport_update_time = now
             self._last_repaint = now
             self._bytes_since_update = 0
             self._last_transport_msg = msg
             self._repaint()
-
-
